@@ -92,9 +92,9 @@ struct fasthistories : public Worker {
             if (c >= 0) {    // ignore unset traps
                 for (m=0; m<mm; m++) {
                     if (binomN==0)
-                        pm0kn[kk * m + k] = gpois (0, Tsk[k] * hk[i3(c, k, m, cc, kk)], 0);
+                        pm0kn[kk * m + k] = gpois (0, Tsk[k] * hk[i3(c, k, m, cc, kk)]);
                     else
-                        pm0kn[kk * m + k] = gbinom (0, Tsk[k], gk[i3(c, k, m, cc, kk)], 0);
+                        pm0kn[kk * m + k] = gbinom (0, Tsk[k], gk[i3(c, k, m, cc, kk)]);
                     pm0n[m] *= pm0kn[kk * m + k];
                 }
             }
@@ -136,12 +136,12 @@ struct fasthistories : public Worker {
                             //     Rprintf("w(n,i) %8.6g, TSKetc %8.6g dpois %8.6g \n", 
                             //         w(n,i), 
                             //         Tsk[k] * hk[i3(c, k, m, cc, kk)],
-                            //         gpois (w(n,i), Tsk[k] * hk[i3(c, k, m, cc, kk)], 0) / pm0ktmp);
+                            //         gpois (w(n,i), Tsk[k] * hk[i3(c, k, m, cc, kk)]) / pm0ktmp);
                             // }
-                            pm[m] *= gpois (w(n,i), Tsk[k] * hk[i3(c, k, m, cc, kk)], 0) / pm0ktmp;
+                            pm[m] *= gpois (w(n,i), Tsk[k] * hk[i3(c, k, m, cc, kk)]) / pm0ktmp;
                         }
                         else {
-                            pm[m] *= gbinom (w(n,i), Tsk[k], gk[i3(c, k, m, cc, kk)], 0) / pm0ktmp;
+                            pm[m] *= gbinom (w(n,i), Tsk[k], gk[i3(c, k, m, cc, kk)]) / pm0ktmp;
                         }
                     }
                     else {
